component {

	property name="mapkitApiService"                inject="mapkitApiService";
	property name="mapkitAuthenticationService"     inject="mapkitAuthenticationService";
	property name="weatherkitAuthenticationService" inject="weatherkitAuthenticationService";

	private function index( event, rc, prc, args={} ) {
		if ( !event.isAdminUser() ) {
			return "";
		}

		args.generatorUrl  = event.buildLink( linkTo="formcontrols.AppleServicesTokenGenerator.generate" );
		args.alertTitle    = translateResource( "formcontrols.appleServicesTokenGenerator:error.title" );
		args.copiedText    = translateResource( "formcontrols.appleServicesTokenGenerator:confirm.copied" );
		args.alertPosition = getSetting( "adminNotificationsPosition" );
		args.service       = args.service ?: "";
		args.type          = ( args.service == "mapkitapi" ) ? "authtoken" : "jwt";

		event.include( "/js/admin/appleServicesTokenGenerator/" );

		return renderView( view="formcontrols/appleServicesTokenGenerator/index", args=args );
	}

	public void function generate( event, rc, prc ) {
		if ( !event.isAdminUser() ) {
			var error = translateResource( "formcontrols.appleServicesTokenGenerator:error.access.denied" );
			event.renderData( type="json", data={ error=error }, statusText=error, statusCode=401 );
			return;
		}

		var expiry     = Int( Val( rc?.expiry ) );
		var expiryUnit = LCase( rc?.unit );
		var service    = rc.service ?: "";
		var response   = {};
		var statusText = "";
		var statusCode = 200;

		if ( expiry <= 0 || !ListFind( "s,n,h,d", expiryUnit ) ) {
			expiry     = 20;
			expiryUnit = "n";
		}

		var expiresIn  = CreateTimeSpan(
			  expiryUnit == "d" ? expiry : 0
			, expiryUnit == "h" ? expiry : 0
			, expiryUnit == "n" ? expiry : 0
			, expiryUnit == "s" ? expiry : 0
		);

		switch( service ) {
			case "mapkitapi":
				try {
					var token = mapkitApiService.getAccessToken();
					response = { token=token };
				} catch( any e ) {
					statusCode = Val( e.errorcode ?: "" ) ? e.errorcode : 500;
					statusText = e.message;
					response   = { error=e.message };
				}
				break;
			case "mapkitjs":
				response = { token=mapkitAuthenticationService.getJwt( expiresIn ) };
				break;
			case "weatherkit":
				response = { token=weatherkitAuthenticationService.getJwt( expiresIn ) };
				break;
			default:
				statusCode = 400;
				statusText = translateResource( "formcontrols.appleServicesTokenGenerator:error.invalid.service" );
				response   = { error=statusText };
		}

		event.renderData( type="json", data=response, statusCode=statusCode, statusText=statusText );
	}

}
