/**
 * @presideService
 * @singleton
 */
component {

	property name="jwt"           inject="jwt@jwtcfml";
	property name="encodingUtils" inject="encodingUtils@jwtcfml";
	property name="utils"         inject="AppleServicesUtils";


// CONSTRUCTOR
	public any function init() {
		variables.JSS         = CreateObject( "java", "java.security.Signature" );
		variables.LEGACY_JAVA = _isLegacyJava();

		return this;
	}

	public string function getJwt( timespan expiresIn=CreateTimeSpan( 0, 0, 20, 0 ) ) {
		var event    = $getRequestContext();
		var prc      = event.getCollection( private=true );
		var domain   = prc._site.domain ?: "";
		var settings = $getPresideCategorySettings( "appleservices" );
		var headers  = { kid=settings.key_id };
		var expires  = utils.addTimeSpanToDate( arguments.expiresIn );
		var payload  = {
			  iss    = settings.team_id
			, iat    = Now()
			, exp    = expires
		};

		if ( Len( domain ) ) {
			payload.origin = domain;
		}

		return jwt.encode( payload=payload, key=settings.private_key, algorithm="ES256", headers=headers );
	}

	public string function signES256( message, key ) {
		var algorithm   = variables.LEGACY_JAVA ? "SHA256withECDSA" : "SHA256withECDSAinP1363Format";
		var jssInstance = variables.JSS.getInstance( algorithm );
		var parsedKey   = encodingUtils.parsePEMEncodedKey( arguments.key );

		jssInstance.initSign( parsedKey );
		jssInstance.update( CharsetDecode( message, "utf-8" ) );

		var sig = jssInstance.sign();
		if ( variables.LEGACY_JAVA ) {
			sig = encodingUtils.convertDERtoP1363( sig, algorithm );
		}

		return encodingUtils.binaryToBase64Url( sig );
	}


// PRIVATE METHODS
	private boolean function _isLegacyJava() {
		var javaFullVersion = CreateObject( "java", "java.lang.System" ).getProperty( "java.version" );
		var javaVersion     = 0;

		if ( Left( javaFullVersion, 2 ) == "1." ) {
			javaVersion = Int( ListGetAt( javaFullVersion, 2, "." ) );
		} else {
			javaVersion = Int( ListFirst( javaFullVersion, "." ) );
		}

		return ( javaVersion < 11 );
	}

}