component accessors=true {

	property name="points"   type="array";
	property name="lines"    type="array";
	property name="polygons" type="array";

	public MapkitGeoJSON function init(
		  array points=[]
		, array lines=[]
		, array polygons=[]
	){
		setPoints( arguments.points )
		setLines( arguments.lines )
		setPolygons( arguments.polygons )

		return this;
	}

	public struct function build() {
		var feature = {};
		var point   = {};
		var key     = "";
		var data    = {
			  type     = "FeatureCollection"
			, features = []
		};

		for( point in getPoints() ) {
			feature = {
				  properties = {}
				, type       = "Feature"
				, geometry   = {
					  coordinates = point.coordinates
					, type        = "Point"
				}
			};
			for( key in point ) {
				if ( key != "coordinates" && !IsNull( point[ key ] ) ) {
					feature.properties[ key ] = point[ key ];
				}
			}

			ArrayAppend( data.features, feature );
		}

		return data;
	}

	public MapkitGeoJSON function addPoint(
		  required array   coordinates
		,          string  title
		,          string  subtitle
		,          string  color
		,          string  glyphText
		,          string  glyphColor
		,          any     glyphImage
		,          string  clusteringIdentifier
		,          boolean animates
		,          boolean selected
	) {
		var point = Duplicate( arguments );
		if ( IsSimpleValue( point.glyphImage ) && Len( point.glyphImage ) ) {
			point.glyphImage = { "1"=point.glyphImage };
		}
		if ( !StructIsEmpty( point.glyphImage ?: {} ) ) {
			point.glyphText = "";
		}
		ArrayAppend( getPoints(), point );

		return this;
	}

	public MapkitGeoJSON function addLine(
		required array coordinates
	) {
		ArrayAppend( getLines(), Duplicate( arguments ) );
		return this;
	}

	public MapkitGeoJSON function addPolygon(
		required array coordinates
	) {
		ArrayAppend( getPolygons(), Duplicate( arguments ) );
		return this;
	}

}