component accessors=true {
	property name="width"       type="numeric" default=0;
	property name="height"      type="numeric" default=0;
	property name="scale"       type="numeric" default=1;
	property name="center"      type="any"     default="";
	property name="zoom"        type="numeric" default="";
	property name="span"        type="string"  default="";
	property name="type"        type="string"  default="";
	property name="darkMode"    type="boolean" default=false;
	property name="showPoi"     type="boolean" default=true;
	property name="language"    type="string"  default="";

	property name="annotations" type="array";
	property name="overlays"    type="array";

	public MapkitSnapshot function init( required any snapshotService, required any utils ){
		variables.snapshotService = arguments.snapshotService;
		variables.utils           = arguments.utils;

		return this;
	}

	public string function getUrl( timespan expiresIn ) {
		return snapshotService.buildSnapshotUrl( params=getParams(), expiresIn=arguments.expiresIn );
	}

	public binary function getBinary() {
		return snapshotService.getSnapshotBinary( getUrl() );
	}

	public struct function getParams() {
		var params = {};

		params.size   = ArrayToList( [ getWidth(), getHeight() ] , "x" );
		params.center = utils.normalizeCoordinate( getCenter() );

		if ( Val( getZoom() ) ) {
			params.z = getZoom();
		}
		if ( ArrayFind( [ 1,2,3 ], getScale() ) ) {
			params.scale = getScale();
		}
		if ( Len( getSpan() ) && getCenter() != "auto" ) {
			params.spn = getSpan();
		}
		if ( Len( getType() ) ) {
			params.t = getType();
		}
		if ( getDarkMode() ) {
			params.colorScheme = "dark";
		}
		if ( !getShowPoi() ) {
			params.poi = 0;
		}
		if ( Len( getLanguage() ) ) {
			params.lang = getLanguage();
		}
		if ( ArrayLen( getAnnotations() ) ) {
			params.annotations = getAnnotations();
		}
		if ( ArrayLen( getOverlays() ) ) {
			params.overlays = getOverlays();
		}

		return params;
	}

	public array function getAnnotations() {
		return variables.annotations ?: [];
	}
	public array function getOverlays() {
		return variables.overlays ?: [];
	}

	public MapkitSnapshot function addAnnotation(
		  required any    point
		,          string color       = ""
		,          string markerStyle = ""
		,          string glyphText   = ""
	) {
		var annotation = {};

		for( var key in [ "point", "color", "markerStyle", "glyphText" ] ) {
			if ( Len( arguments[ key ] ) ) {
				annotation[ key ] = key=="point" ? utils.normalizeCoordinate( arguments[ key ] ) : arguments[ key ];
			}
		}

		variables.annotations = variables.annotations ?: [];
		ArrayAppend( variables.annotations, annotation );

		return this;
	}

	public MapkitSnapshot function addOverlay(
		  required array   points
		,          string  strokeColor = ""
		,          numeric lineWidth   = 0
		,          array   lineDash    = []
	) {
		var overlay = {};

		overlay.points = [];
		for( var point in arguments.points ) {
			ArrayAppend( overlay.points, utils.normalizeCoordinate( point ) );
		}

		if ( Len( arguments.strokeColor ) ) {
			overlay.strokeColor = arguments.strokeColor;
		}

		overlay.lineWidth = arguments.lineWidth;

		if ( ArrayLen( arguments.lineDash ) ) {
			overlay.lineDash = arguments.lineDash;
		}

		variables.overlays = variables.overlays ?: [];
		ArrayAppend( variables.overlays, overlay );

		return this;
	}

	public boolean function validate() {
		return ArrayIsEmpty( getErrors() );
	}

	public array function getErrors() {
		return snapshotService.validationErrors( this );
	}

}